"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const macro_1 = require("remax/macro");
const lodash_1 = require("lodash");
const getEntries_1 = require("../getEntries");
class API {
    constructor() {
        this.plugins = [];
        this.adapter = {
            name: '',
            packageName: '',
            options: {},
        };
        this.meta = {
            template: {
                extension: '',
                tag: '',
                src: '',
            },
            style: '',
            jsHelper: {
                extension: '',
                tag: '',
                src: '',
            },
            include: {
                tag: '',
                src: '',
            },
        };
    }
    extendsCLI(options) {
        let { cli } = options;
        this.plugins.forEach(plugin => {
            if (typeof plugin.extendsCLI === 'function') {
                cli = plugin.extendsCLI(Object.assign(Object.assign({}, options), { cli }));
            }
        });
        return cli;
    }
    getMeta() {
        let meta = {
            template: {
                extension: '',
                tag: '',
                src: '',
            },
            style: '',
            jsHelper: {
                extension: '',
                tag: '',
                src: '',
            },
            include: {
                tag: '',
                src: '',
            },
            ejs: {
                page: '',
            },
        };
        this.plugins.forEach(plugin => {
            meta = lodash_1.merge(meta, plugin.meta || {});
        });
        return meta;
    }
    getHostComponents() {
        return macro_1.hostComponents;
    }
    getEntries(entries, appManifest, remaxOptions) {
        this.plugins.forEach(plugin => {
            if (typeof plugin.getEntries === 'function') {
                const currentEntries = plugin.getEntries({
                    remaxOptions,
                    appManifest,
                    getEntryPath: (entryPath) => getEntries_1.searchFile(entryPath, true),
                });
                entries.app = currentEntries.app || entries.app;
                entries.pages = [...entries.pages, ...currentEntries.pages];
                entries.images = [...entries.images, ...currentEntries.images];
            }
        });
        return entries;
    }
    processProps(componentName, props, additional, node) {
        let nextProps = props;
        this.plugins.forEach(plugin => {
            if (typeof plugin.processProps === 'function') {
                nextProps = plugin.processProps({
                    componentName,
                    props: nextProps,
                    additional,
                    node,
                });
            }
        });
        return nextProps;
    }
    shouldHostComponentRegister(componentName, phase, additional) {
        return this.plugins.reduce((result, plugin) => {
            if (typeof plugin.shouldHostComponentRegister === 'function') {
                return plugin.shouldHostComponentRegister({
                    componentName,
                    additional,
                    phase,
                });
            }
            return result;
        }, true);
    }
    extendsRollupConfig(options) {
        let { rollupConfig } = options;
        this.plugins.forEach(plugin => {
            if (typeof plugin.extendsRollupConfig === 'function') {
                rollupConfig = plugin.extendsRollupConfig(Object.assign(Object.assign({}, options), { rollupConfig }));
            }
        });
        return rollupConfig;
    }
    registerAdapterPlugins(adapterName) {
        this.adapter.name = adapterName;
        this.adapter.packageName = 'remax-' + adapterName;
        const packagePath = this.adapter.packageName + '/node';
        delete require.cache[packagePath];
        let plugin = require(packagePath).default || require(packagePath);
        plugin = typeof plugin === 'function' ? plugin() : plugin;
        this.registerHostComponents(plugin.hostComponents);
        this.plugins.push(plugin);
    }
    registerNodePlugins(remaxConfig) {
        remaxConfig.plugins.forEach(plugin => {
            if (plugin) {
                this.registerHostComponents(plugin.hostComponents);
                this.plugins.push(plugin);
            }
        });
    }
    registerHostComponents(components) {
        if (!components) {
            return;
        }
        for (const key of components.keys()) {
            macro_1.hostComponents.set(key, components.get(key));
        }
    }
}
exports.default = new API();
