"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const t = __importStar(require("@babel/types"));
const lodash_1 = require("lodash");
const API_1 = __importDefault(require("../../API"));
const importers = new Map();
function convertComponents(importers) {
    const components = [...importers.values()].reduce((prev, components) => {
        for (const component of components.values()) {
            const com = prev.find(com => com.id === component.id);
            if (com) {
                com.props = [...new Set([...com.props, ...component.props])].sort();
                if (com.pages && component.pages) {
                    com.pages = new Set([...com.pages, ...component.pages]);
                }
                continue;
            }
            prev.push(lodash_1.cloneDeep(component));
        }
        return prev;
    }, []);
    return components.map(c => (Object.assign(Object.assign({}, c), { props: [...c.props] })));
}
exports.convertComponents = convertComponents;
function addToComponentCollection(component, importers) {
    const components = importers.get(component.importer);
    if (!components) {
        importers.set(component.importer, new Map([[component.id, component]]));
        return;
    }
    const com = components.get(component.id);
    if (!com) {
        components.set(component.id, component);
        return;
    }
    components.set(component.id, Object.assign(Object.assign({}, component), { props: [...new Set([...component.props, ...com.props])].sort() }));
}
exports.addToComponentCollection = addToComponentCollection;
function registerProps(componentName, node) {
    const hostComponent = API_1.default.getHostComponents().get(componentName);
    if (!hostComponent) {
        return;
    }
    const usedProps = API_1.default.processProps(componentName, hostComponent.props.slice(), hostComponent.additional, node);
    if (node) {
        node.openingElement.attributes.forEach(attr => {
            if (t.isJSXSpreadAttribute(attr)) {
                return;
            }
            const propName = attr.name.name;
            usedProps.push(propName);
        });
    }
    usedProps.push('data-rid');
    return [
        ...new Set(usedProps
            // 无需收集 slot 字段
            .filter(p => p !== 'slot')
            .filter(Boolean)
            .map(prop => { var _a, _b; return ((_b = (_a = hostComponent) === null || _a === void 0 ? void 0 : _a.alias) === null || _b === void 0 ? void 0 : _b[prop]) || prop; })),
    ].sort();
}
function registerComponent({ componentName, node, importer = '', phase, }, additional) {
    if (!API_1.default.shouldHostComponentRegister(componentName, phase, additional)) {
        return;
    }
    const props = registerProps(componentName, node);
    if (!props) {
        return;
    }
    const component = {
        id: componentName,
        props,
        importer,
    };
    addToComponentCollection(component, importers);
}
exports.default = () => {
    importers.clear();
    return {
        pre(state) {
            importers.delete(state.opts.filename);
        },
        visitor: {
            ImportDeclaration(path, state) {
                const node = path.node;
                if (!node.source.value.startsWith('remax/')) {
                    return;
                }
                node.specifiers.forEach(specifier => {
                    if (t.isImportSpecifier(specifier)) {
                        const componentName = specifier.imported.name;
                        const id = lodash_1.kebabCase(componentName);
                        registerComponent({
                            componentName: id,
                            importer: state.file.opts.filename,
                            phase: 'import',
                        });
                    }
                });
            },
            JSXElement(path, state) {
                const node = path.node;
                if (t.isJSXIdentifier(node.openingElement.name)) {
                    const tagName = node.openingElement.name.name;
                    const binding = path.scope.getBinding(tagName);
                    if (!binding) {
                        return;
                    }
                    const componentPath = binding.path;
                    if (!componentPath ||
                        !t.isImportSpecifier(componentPath.node) ||
                        !t.isImportDeclaration(componentPath.parent)) {
                        return;
                    }
                    const componentName = componentPath.node
                        .imported.name;
                    const id = lodash_1.kebabCase(componentName);
                    registerComponent({
                        componentName: id,
                        node,
                        importer: state.file.opts.filename,
                        phase: 'jsx',
                    });
                }
            },
        },
    };
};
function getComponents() {
    API_1.default.getHostComponents().forEach((component, componentName) => {
        registerComponent({
            componentName,
            phase: 'extra',
        }, component.additional);
    });
    return convertComponents(importers);
}
exports.getComponents = getComponents;
