"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const acorn_walk_1 = require("acorn-walk");
const magic_string_1 = __importDefault(require("magic-string"));
/**
 * rollup-plugin-commonjs 把 regenerator-runtime  变成了 es module
 * 然后 es module 默认就是 strict mode
 * 支付宝小程序用 webpack 打包的时候默认会给 es module 加个 use strict。
 */
function fixRegeneratorRuntime() {
    return {
        name: 'fix-regenerator-runetime',
        transform(code) {
            const magicString = new magic_string_1.default(code);
            const ast = this.parse(code, {
                sourceType: 'module',
            });
            const remove = (node) => {
                if (node.callee.type === 'CallExpression' &&
                    node.callee.callee.name === 'Function' &&
                    node.callee.arguments.length === 2 &&
                    node.callee.arguments[0].value === 'r' &&
                    node.callee.arguments[1].value === 'regeneratorRuntime = r') {
                    magicString.remove(node.start, node.end);
                }
            };
            acorn_walk_1.simple(ast, {
                CallExpression: remove,
            });
            return {
                code: magicString.toString(),
                map: magicString.generateMap().toString(),
            };
        },
    };
}
exports.default = fixRegeneratorRuntime;
