"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const dot_prop_1 = require("dot-prop");
const lodash_1 = require("lodash");
const t = __importStar(require("@babel/types"));
const path = __importStar(require("path"));
const util_1 = require("./util");
const components_1 = require("../components");
const importers = new Map();
exports.getKebabCaseName = (sourcePath) => lodash_1.kebabCase(path.basename(path.dirname(sourcePath)));
const nativeIds = new Map();
const getHashId = (sourcePath, id) => {
    const sourcePaths = nativeIds.get(id);
    if (sourcePaths) {
        const index = sourcePaths.findIndex(source => source === sourcePath);
        if (index >= 0) {
            return `${id}-${index}`;
        }
        sourcePaths.push(sourcePath);
        return `${id}-${sourcePaths.length - 1}`;
    }
    nativeIds.set(id, [sourcePath]);
    return `${id}-0`;
};
exports.default = (options) => {
    importers.clear();
    return () => ({
        pre(state) {
            importers.delete(state.opts.filename);
        },
        visitor: {
            ImportDeclaration(path, state) {
                const importer = state.file.opts.filename;
                const node = path.node;
                const sourcePath = util_1.getSourcePath(options, node.source.value, importer);
                if (!util_1.isNativeComponent(sourcePath)) {
                    return;
                }
                const id = exports.getKebabCaseName(sourcePath);
                const component = {
                    id: sourcePath,
                    props: [],
                    importer,
                    hashId: getHashId(sourcePath, id),
                    pages: new Set([]),
                };
                components_1.addToComponentCollection(component, importers);
            },
            JSXElement(nodePath, state) {
                const importer = state.file.opts.filename;
                const node = nodePath.node;
                if (t.isJSXIdentifier(node.openingElement.name)) {
                    const tagName = node.openingElement.name.name;
                    const binding = nodePath.scope.getBinding(tagName);
                    if (!binding) {
                        return;
                    }
                    const componentPath = binding.path;
                    if (!componentPath ||
                        !t.isImportDefaultSpecifier(componentPath.node) ||
                        !t.isImportDeclaration(componentPath.parent)) {
                        return;
                    }
                    const source = componentPath.parent.source.value;
                    const sourcePath = util_1.getSourcePath(options, source, importer);
                    if (!util_1.isNativeComponent(sourcePath) &&
                        !util_1.isPluginComponent(sourcePath, options)) {
                        return;
                    }
                    const id = exports.getKebabCaseName(sourcePath);
                    const usedProps = node.openingElement.attributes.map(e => {
                        const propName = dot_prop_1.get(e, 'name.name');
                        return propName;
                    });
                    const props = usedProps
                        .filter(Boolean)
                        // 剔除 ref，在 axml 特殊处理
                        .filter(k => k !== 'ref')
                        .map(prop => prop.replace('className', 'class'));
                    const component = {
                        id: sourcePath,
                        props: [...new Set(props)].sort(),
                        importer,
                        hashId: getHashId(sourcePath, id),
                        pages: new Set([]),
                    };
                    components_1.addToComponentCollection(component, importers);
                }
            },
        },
    });
};
exports.getNativeComponents = () => lodash_1.sortBy(components_1.convertComponents(importers).map(component => {
    return Object.assign(Object.assign({}, component), { type: 'native', id: component.hashId, sourcePath: component.id });
}), 'id');
exports.getImporters = () => importers;
