"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const magic_string_1 = __importDefault(require("magic-string"));
const path = __importStar(require("path"));
const acorn_walk_1 = require("acorn-walk");
const fs_1 = require("fs");
const style_1 = __importStar(require("./style"));
const json_1 = __importStar(require("./json"));
const tempate_1 = __importStar(require("./tempate"));
const jsHelper_1 = __importStar(require("./jsHelper"));
const util_1 = require("./util");
const winPath_1 = __importDefault(require("../../../winPath"));
const usingComponents_1 = __importDefault(require("./usingComponents"));
const babelPlugin_1 = require("./babelPlugin");
const getEntries_1 = require("../../../getEntries");
const getFiles = () => [
    ...style_1.getcssPaths(),
    ...json_1.getjsonPaths(),
    ...tempate_1.getTemplatePaths(),
    ...jsHelper_1.getJsHelpers(),
];
exports.default = (options, pages) => {
    return {
        name: 'nativeComponents',
        load(id) {
            if (util_1.isNativeComponent(id)) {
                jsHelper_1.default(id);
                style_1.default(id);
                json_1.default(id);
                tempate_1.default(id);
                usingComponents_1.default(id, options, this);
                getFiles().forEach(file => {
                    this.addWatchFile(file);
                });
            }
            return null;
        },
        transform(code, id) {
            const importers = babelPlugin_1.getImporters();
            const importer = importers.get(id);
            const ast = this.parse(code, {
                sourceType: 'module',
            });
            const magicString = new magic_string_1.default(code);
            if (!importer) {
                return {
                    code: magicString.toString(),
                    map: magicString.generateMap(),
                };
            }
            let helperImported = false;
            const extract = (node) => {
                const source = lodash_1.get(node, 'source.value');
                const name = lodash_1.get(node, 'specifiers[0].local.name');
                const componentPath = util_1.getSourcePath(options, source, id);
                const component = importer.get(componentPath);
                if (!component) {
                    return;
                }
                if (!util_1.isPluginComponent(componentPath, options)) {
                    this.emitFile({
                        id: path.relative(options.cwd, componentPath),
                        type: 'chunk',
                    });
                }
                magicString.remove(node.start, node.end);
                if (!helperImported) {
                    magicString.prepend("import { unstable_createNativeComponent } from 'remax';\n");
                    helperImported = true;
                }
                const exportStr = `var ${name} = unstable_createNativeComponent('${component.hashId}')\n`;
                magicString.prepend(exportStr);
            };
            acorn_walk_1.simple(ast, {
                ImportDeclaration: extract,
            });
            return {
                code: magicString.toString(),
                map: magicString.generateMap(),
            };
        },
        generateBundle(_, bundle) {
            const importers = babelPlugin_1.getImporters();
            const collected = new Map();
            const collectPages = (page, importer) => {
                const pageCollected = collected.get(page);
                if (pageCollected) {
                    if (pageCollected.has(importer)) {
                        return;
                    }
                    pageCollected.add(importer);
                }
                else {
                    collected.set(page, new Set([importer]));
                }
                const nativeImporter = importers.get(getEntries_1.searchFile(path.join(options.cwd, importer).replace(path.extname(importer), '')));
                if (nativeImporter) {
                    [...nativeImporter.values()].forEach(component => {
                        component.pages = new Set([...component.pages, page]);
                    });
                }
                importer = winPath_1.default(importer)
                    .replace(/node_modules/g, 'npm')
                    .replace(new RegExp(`^${options.rootDir}/`), '')
                    .replace(/@/g, '_')
                    .replace(path.extname(importer), '.js');
                const { imports } = bundle[importer] || {
                    imports: [],
                };
                for (const file of imports) {
                    collectPages(page, file);
                }
            };
            for (const key of pages) {
                collectPages(key, path.relative(options.cwd, key));
            }
            getFiles().forEach(id => {
                var _a;
                const bundleFileName = winPath_1.default(path.relative(options.cwd, id).replace(/node_modules/g, 'npm'))
                    .replace(/^src\//, '')
                    .replace(/@/g, '_');
                const source = fs_1.readFileSync(id);
                if (((_a = this.cache.get(bundleFileName)) === null || _a === void 0 ? void 0 : _a.toString()) === source.toString()) {
                    return;
                }
                this.cache.set(bundleFileName, source);
                this.emitFile({
                    fileName: bundleFileName,
                    type: 'asset',
                    source,
                });
            });
        },
    };
};
