"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const babelParser = __importStar(require("@babel/parser"));
const traverse_1 = __importDefault(require("@babel/traverse"));
const htmlparser2 = __importStar(require("htmlparser2"));
const lodash_1 = require("lodash");
const API_1 = __importDefault(require("../../../API"));
const util_1 = require("./util");
const output_1 = require("../../utils/output");
const jsHelpers = [];
const walk = (jsHelperPath) => {
    const jsHelperContent = util_1.readFile(jsHelperPath);
    const ast = babelParser.parse(jsHelperContent, {
        sourceType: 'module',
    });
    const extract = ({ node }) => {
        const importPath = (lodash_1.get(node, 'callee.name') === 'require'
            ? lodash_1.get(node, 'arguments[0].value')
            : '') || lodash_1.get(node, 'source.value');
        if (!importPath) {
            return;
        }
        const absolutePath = util_1.getPath(jsHelperPath, importPath);
        util_1.pushArray(jsHelpers, absolutePath);
        walk(absolutePath);
    };
    traverse_1.default(ast, {
        CallExpression: extract,
        ImportDeclaration: extract,
    });
};
const parseTemplate = (filePath, jsHelper) => {
    const parser = new htmlparser2.Parser({});
    const { tag, src } = jsHelper;
    const content = util_1.readFile(filePath);
    parser._cbs.onopentag = (name, attrs) => {
        if (name === tag && attrs[src]) {
            const jsHelperPath = util_1.getPath(filePath, attrs[src]);
            if (!fs_1.default.existsSync(jsHelperPath)) {
                output_1.output(`\n🚨 文件 ${jsHelperPath} 不存在`, 'red');
                return;
            }
            walk(jsHelperPath);
            util_1.pushArray(jsHelpers, jsHelperPath);
        }
    };
    parser.reset();
    parser.write(content);
    parser.end();
};
function jsHelper(id) {
    const { jsHelper, template } = API_1.default.getMeta();
    if (!jsHelper) {
        return;
    }
    const templatePath = id.replace(/\.js$/, template.extension);
    parseTemplate(templatePath, jsHelper);
}
exports.default = jsHelper;
exports.getJsHelpers = () => jsHelpers;
