"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const resolve_1 = __importDefault(require("resolve"));
const extensions_1 = __importDefault(require("../../../extensions"));
const alias_1 = __importDefault(require("../alias"));
const winPath_1 = __importDefault(require("../../../winPath"));
const getEntries_1 = require("../../../getEntries");
exports.pushArray = (arr, path) => {
    if (arr.includes(path)) {
        return;
    }
    arr.push(path);
};
exports.isPluginComponent = (sourcePath, options) => {
    if (!sourcePath.startsWith('plugin://')) {
        return false;
    }
    const { plugins, subpackages = [], subPackages = [] } = getEntries_1.getAppConfig(options);
    const subs = [...subpackages, ...subPackages];
    if (!plugins && !subs.some(sub => sub.plugins)) {
        return false;
    }
    const [, pluginName = ''] = /^plugin:\/\/(\w+)\//.exec(sourcePath) || [];
    if (!pluginName) {
        return false;
    }
    if ((plugins && plugins[pluginName]) ||
        subs.find(sub => sub.plugins && sub.plugins[pluginName])) {
        return true;
    }
    return false;
};
exports.getSourcePath = (options, source, importer) => {
    let sourcePath = alias_1.default(options).resolveId(source, importer) || source;
    // ignore rollup's virtual modules
    if (sourcePath.startsWith('\0')) {
        return sourcePath;
    }
    if (!exports.isPluginComponent(sourcePath, options)) {
        sourcePath = resolve_1.default.sync(sourcePath, {
            extensions: extensions_1.default,
            basedir: path_1.default.dirname(importer),
        });
    }
    return sourcePath;
};
exports.isNativeComponent = (sourcePath) => {
    if (path_1.default.extname(sourcePath) !== '.js') {
        return false;
    }
    const sourceJsonPath = sourcePath.replace(/\.js$/, '.json');
    if (!fs.existsSync(sourceJsonPath)) {
        return false;
    }
    return require(sourceJsonPath).component;
};
exports.getPath = (from, to) => {
    return winPath_1.default(path_1.default.resolve(path_1.default.dirname(from), to));
};
exports.readFile = (filePath) => {
    return fs.readFileSync(filePath).toString();
};
