"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const acorn_walk_1 = require("acorn-walk");
const magic_string_1 = __importDefault(require("magic-string"));
const winPath_1 = __importDefault(require("../../winPath"));
var NodeType;
(function (NodeType) {
    NodeType["Literal"] = "Literal";
    NodeType["CallExpresssion"] = "CallExpression";
    NodeType["Identifier"] = "Identifier";
    NodeType["ImportDeclaration"] = "ImportDeclaration";
    NodeType["ExportNamedDeclaration"] = "ExportNamedDeclaration";
    NodeType["ExportAllDeclaration"] = "ExportAllDeclaration";
})(NodeType || (NodeType = {}));
const PARENT_DIR_PATTERN = /^\.\.\//;
function isAsset(module) {
    return !!module && module.isAsset;
}
exports.isAsset = isAsset;
function getImportSource(node) {
    if (node.type !== NodeType.ImportDeclaration ||
        node.source.type !== NodeType.Literal) {
        return false;
    }
    return node.source;
}
exports.getImportSource = getImportSource;
function getExportSource(node) {
    if (node.type !== NodeType.ExportNamedDeclaration ||
        !node.source ||
        node.source.type !== NodeType.Literal) {
        return false;
    }
    return node.source;
}
function isEmpty(array) {
    return !array || array.length === 0;
}
function getRequireSource(node) {
    if (node.type !== NodeType.CallExpresssion) {
        return false;
    }
    if (node.callee.type !== NodeType.Identifier || isEmpty(node.arguments)) {
        return false;
    }
    const args = node.arguments;
    if (node.callee.name !== 'require' || args[0].type !== NodeType.Literal) {
        return false;
    }
    return args[0];
}
exports.getRequireSource = getRequireSource;
function removeSrc(options) {
    const PREFIX_SRC_PATTERN = new RegExp(`^${options.rootDir}/`);
    const rewrite = (input) => {
        return input.replace(PREFIX_SRC_PATTERN, '');
    };
    const isInsideSrc = (file, req) => {
        return winPath_1.default(path.resolve(path.dirname(file), req).replace(process.cwd(), '')).startsWith(`/${options.rootDir}/`);
    };
    return {
        name: 'remove-src',
        generateBundle(_, bundle) {
            const files = Object.keys(bundle);
            files.map(file => {
                if (PREFIX_SRC_PATTERN.test(file)) {
                    const module = bundle[file];
                    if (isAsset(module)) {
                        return;
                    }
                    module.fileName = rewrite(module.fileName);
                    module.facadeModuleId = rewrite(module.fileName);
                    if (module.code) {
                        const magicString = new magic_string_1.default(module.code);
                        const ast = this.parse(module.code, {
                            sourceType: 'module',
                        });
                        const extract = (node) => {
                            const req = getRequireSource(node) ||
                                getImportSource(node) ||
                                getExportSource(node);
                            if (req) {
                                const { start, end } = req;
                                const distance = req.value
                                    .split('/')
                                    .filter((d) => d === '..').length;
                                const targetDistance = winPath_1.default(path.relative(path.dirname(file), options.rootDir)).split('/').length;
                                if (isInsideSrc(file, req.value)) {
                                    return;
                                }
                                if (distance == 1 && targetDistance === 1) {
                                    // app.js
                                    const newPath = req.value.replace(PARENT_DIR_PATTERN, './');
                                    magicString.overwrite(start, end, `'${newPath}'`);
                                }
                                else if (distance > targetDistance) {
                                    // page
                                    const newPath = req.value.replace(PARENT_DIR_PATTERN, '');
                                    magicString.overwrite(start, end, `'${newPath}'`);
                                }
                            }
                        };
                        acorn_walk_1.simple(ast, {
                            ImportDeclaration: extract,
                            CallExpression: extract,
                            ExportNamedDeclaration: extract,
                        });
                        module.code = magicString.toString();
                    }
                    delete bundle[file];
                    bundle[rewrite(file)] = module;
                }
            });
        },
    };
}
exports.default = removeSrc;
