"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const rollup_pluginutils_1 = require("rollup-pluginutils");
const acorn_walk_1 = require("acorn-walk");
const magic_string_1 = __importDefault(require("magic-string"));
var NodeType;
(function (NodeType) {
    NodeType["Literal"] = "Literal";
    NodeType["CallExpresssion"] = "CallExpression";
    NodeType["Identifier"] = "Identifier";
    NodeType["ImportDeclaration"] = "ImportDeclaration";
    NodeType["ExportNamedDeclaration"] = "ExportNamedDeclaration";
    NodeType["ExportAllDeclaration"] = "ExportAllDeclaration";
})(NodeType || (NodeType = {}));
function isEmpty(array) {
    return !array || array.length === 0;
}
exports.isEmpty = isEmpty;
function getRequireSource(node) {
    if (node.type !== NodeType.CallExpresssion) {
        return false;
    }
    if (node.callee.type !== NodeType.Identifier || isEmpty(node.arguments)) {
        return false;
    }
    const args = node.arguments;
    if (node.callee.name !== 'require' || args[0].type !== NodeType.Literal) {
        return false;
    }
    return args[0];
}
exports.getRequireSource = getRequireSource;
function getImportSource(node) {
    if (node.type !== NodeType.ImportDeclaration ||
        node.source.type !== NodeType.Literal) {
        return false;
    }
    return node.source;
}
exports.getImportSource = getImportSource;
function getExportSource(node) {
    const exportNodes = [
        NodeType.ExportAllDeclaration,
        NodeType.ExportNamedDeclaration,
    ];
    if (!exportNodes.includes(node.type) ||
        !node.source ||
        node.source.type !== NodeType.Literal) {
        return false;
    }
    return node.source;
}
exports.getExportSource = getExportSource;
function rewrite(input, map) {
    return map(input);
}
exports.rewrite = rewrite;
function rename(options) {
    const filter = rollup_pluginutils_1.createFilter(options.include, options.exclude);
    const sourceMaps = options.sourceMap !== false;
    return {
        name: 'rename-rollup',
        generateBundle(_, bundle) {
            const files = Object.entries(bundle);
            for (const [key, file] of files) {
                if (!filter(file.facadeModuleId) &&
                    !(options.matchAll && /\0/.test(file.facadeModuleId)) &&
                    file.fileName !== 'app.css') {
                    continue;
                }
                file.facadeModuleId =
                    rewrite(file.facadeModuleId, options.map) || file.facadeModuleId;
                file.fileName = rewrite(file.fileName, options.map) || file.fileName;
                if (file.imports) {
                    file.imports.map((imported) => {
                        if (!filter(imported)) {
                            return imported;
                        }
                        return rewrite(imported, options.map) || imported;
                    });
                }
                if (file.code) {
                    const magicString = new magic_string_1.default(file.code);
                    const ast = this.parse(file.code, {
                        sourceType: 'module',
                    });
                    const extract = (node) => {
                        const req = getRequireSource(node) ||
                            getImportSource(node) ||
                            getExportSource(node);
                        if (req) {
                            const { start, end } = req;
                            const newPath = rewrite(req.value, options.map);
                            magicString.overwrite(start, end, `'${newPath}'`);
                        }
                    };
                    acorn_walk_1.simple(ast, {
                        ImportDeclaration: extract,
                        CallExpression: extract,
                        ExportAllDeclaration: extract,
                        ExportNamedDeclaration: extract,
                    });
                    if (sourceMaps) {
                        file.map = magicString.generateMap();
                    }
                    file.code = magicString.toString();
                }
                delete bundle[key];
                bundle[rewrite(key, options.map) || key] = file;
            }
        },
    };
}
exports.default = rename;
