"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const magic_string_1 = __importDefault(require("magic-string"));
const winPath_1 = __importDefault(require("../../winPath"));
function stub(options = {}) {
    return {
        name: 'stub',
        transform(code, id) {
            if (options.modules) {
                if (options.modules.every(module => winPath_1.default(id).indexOf(winPath_1.default(module)) === -1)) {
                    return null;
                }
            }
            let ast = this.parse(code, {
                sourceType: 'module',
            });
            const exports = [];
            ast.body.forEach((node) => {
                if (node.type === 'ExportDefaultDeclaration') {
                    exports.push(`\nexport default function() {};`);
                }
                else if (node.type === 'ExportNamedDeclaration') {
                    if (node.declaration == null) {
                        let content = '';
                        node.specifiers.forEach((specifier) => {
                            content += `\nexport var ${specifier.exported.name} = function() {};`;
                        });
                        exports.push(content);
                    }
                    else if (node.declaration.type === 'VariableDeclaration') {
                        node.declaration.declarations.forEach((declaration) => {
                            exports.push(`\nexport var ${declaration.id.name} = function() {};`);
                        });
                    }
                    else if (node.declaration.type === 'FunctionDeclaration') {
                        exports.push(`\nexport var ${node.declaration.id.name} = function() {};`);
                    }
                    else if (node.declaration.type === 'ClassDeclaration') {
                        exports.push(`\nexport var ${node.declaration.id.name} = function() {};`);
                    }
                }
            });
            const magicString = new magic_string_1.default(exports.join(''));
            code = magicString.toString();
            const map = options.sourceMap ? magicString.generateMap() : null;
            ast = this.parse(code, {
                sourceType: 'module',
            });
            return { code, map };
        },
    };
}
exports.default = stub;
