"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const acorn_1 = require("acorn");
const lodash_1 = require("lodash");
const components_1 = require("./components");
const ejs_1 = __importDefault(require("ejs"));
const API_1 = __importDefault(require("../../API"));
const UNSAFE_wechatTemplateDepth_1 = require("../../defaultOptions/UNSAFE_wechatTemplateDepth");
const readManifest_1 = __importDefault(require("../../readManifest"));
const getEntries_1 = __importDefault(require("../../getEntries"));
const winPath_1 = __importDefault(require("../../winPath"));
const babelPlugin_1 = require("./nativeComponents/babelPlugin");
function pageUID(pagePath) {
    return pagePath.replace('/', '_');
}
async function createTemplate(pageFile, options, meta) {
    const fileName = `${path.dirname(pageFile)}/${path.basename(pageFile, path.extname(pageFile))}${meta.template.extension}`;
    const renderOptions = {
        baseTemplate: winPath_1.default(path.relative(path.dirname(pageFile), `base${meta.template.extension}`)),
    };
    if (meta.jsHelper) {
        renderOptions.jsHelper = `./${pageUID(pageFile)}_helper${meta.jsHelper.extension}`;
    }
    const components = lodash_1.sortBy(components_1.getComponents().concat(Object.values(babelPlugin_1.getNativeComponents())), 'id');
    const hostComponents = API_1.default.getHostComponents();
    let code = await ejs_1.default.renderFile(meta.ejs.page, Object.assign(Object.assign({}, renderOptions), { components, viewComponent: {
            props: [...new Set(hostComponents.get('view').props)].sort(),
        } }), {
        rmWhitespace: options.compressTemplate,
    });
    // TODO 用 uglify 替代 compressTemplate
    /* istanbul ignore next */
    if (options.compressTemplate) {
        code = code.replace(/^\s*$(?:\r\n?|\n)/gm, '').replace(/\r\n|\n/g, ' ');
    }
    return {
        type: 'asset',
        fileName,
        source: code,
    };
}
async function createHelperFile(pageFile, meta) {
    if (!meta.jsHelper || !meta.ejs.jsHelper) {
        return null;
    }
    const code = await ejs_1.default.renderFile(meta.ejs.jsHelper, {
        target: API_1.default.adapter.name,
    });
    return {
        type: 'asset',
        fileName: winPath_1.default(path.join(path.dirname(pageFile), `${pageUID(pageFile)}_helper${meta.jsHelper.extension}`)),
        source: code,
    };
}
async function createBaseTemplate(options, meta) {
    if (!meta.ejs.base) {
        return null;
    }
    const hostComponents = API_1.default.getHostComponents();
    const components = lodash_1.sortBy(components_1.getComponents().concat(Object.values(babelPlugin_1.getNativeComponents())), 'id');
    let code = await ejs_1.default.renderFile(meta.ejs.base, {
        components,
        depth: UNSAFE_wechatTemplateDepth_1.ensureDepth(options.UNSAFE_wechatTemplateDepth),
        viewComponent: {
            props: [...new Set(hostComponents.get('view').props)].sort(),
        },
    }, {
        rmWhitespace: options.compressTemplate,
    });
    // TODO 用 uglify 替代 compressTemplate
    /* istanbul ignore next */
    if (options.compressTemplate) {
        code = code.replace(/^\s*$(?:\r\n?|\n)/gm, '').replace(/\r\n|\n/g, ' ');
    }
    return {
        type: 'asset',
        fileName: `base${meta.template.extension}`,
        source: code,
    };
}
function createAppManifest(options, context) {
    const config = context
        ? Object.assign(Object.assign({}, context.app), { pages: context.pages.map(p => p.path) }) : readManifest_1.default(path.resolve(options.cwd, `${options.rootDir}/app.config`), API_1.default.adapter.name);
    return {
        type: 'asset',
        fileName: 'app.json',
        source: JSON.stringify(config, null, 2),
    };
}
function createPageUsingComponents(page, configFilePath, options) {
    const nativeComponents = babelPlugin_1.getNativeComponents();
    const usingComponents = {};
    for (const { id, sourcePath, pages } of nativeComponents) {
        if (!pages.has(page)) {
            continue;
        }
        if (sourcePath.startsWith('plugin://')) {
            usingComponents[id] = sourcePath;
            continue;
        }
        usingComponents[id] = winPath_1.default(path
            .relative(path.dirname(configFilePath), sourcePath
            .replace(/node_modules/, `${options.rootDir}/npm`)
            .replace(/node_modules/g, 'npm'))
            .replace(/\.js$/, '')
            .replace(/@/g, '_'));
    }
    return usingComponents;
}
function createPageManifest(options, file, page, context) {
    const configFile = file.replace(/\.(js|jsx|ts|tsx)$/, '.config');
    const manifestFile = file.replace(/\.(js|jsx|ts|tsx)$/, '.json');
    const configFilePath = path.resolve(options.cwd, path.join(options.rootDir, configFile));
    const usingComponents = createPageUsingComponents(page, configFilePath, options);
    const config = readManifest_1.default(configFilePath, API_1.default.adapter.name);
    config.usingComponents = Object.assign(Object.assign({}, (config.usingComponents || {})), usingComponents);
    if (context) {
        const pageConfig = context.pages.find((p) => p.path === page.path);
        if (pageConfig) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            const { path } = pageConfig, config = __rest(pageConfig, ["path"]);
            config.usingComponents = Object.assign(Object.assign({}, (config.usingComponents || {})), usingComponents);
            return {
                type: 'asset',
                fileName: manifestFile,
                source: JSON.stringify(config, null, 2),
            };
        }
    }
    return {
        type: 'asset',
        fileName: manifestFile,
        source: JSON.stringify(config, null, 2),
    };
}
function isRemaxEntry(chunk) {
    if (!chunk.isEntry) {
        return false;
    }
    const ast = acorn_1.parse(chunk.code, {
        sourceType: 'module',
    });
    return ast.body.every((node) => {
        // 检查是不是原生写法
        if (node.type === 'ExpressionStatement' &&
            node.expression.type === 'CallExpression' &&
            node.expression.callee.type === 'Identifier' &&
            node.expression.callee.name === 'Page' &&
            node.expression.arguments.length > 0 &&
            node.expression.arguments[0].type === 'ObjectExpression') {
            return false;
        }
        return true;
    });
}
function template(options, context) {
    return {
        name: 'template',
        async generateBundle(_, bundle, isWrite) {
            var _a;
            const meta = API_1.default.getMeta();
            const templateAssets = [];
            // app.json
            const manifest = createAppManifest(options, context);
            if (((_a = this.cache.get(manifest.fileName)) === null || _a === void 0 ? void 0 : _a.toString()) !==
                manifest.source.toString()) {
                this.cache.set(manifest.fileName, manifest.source);
                templateAssets.push(manifest);
            }
            const template = await createBaseTemplate(options, meta);
            if (template) {
                templateAssets.push(template);
            }
            const entries = getEntries_1.default(options, context);
            const { pages } = entries;
            const files = Object.keys(bundle);
            await Promise.all(files.map(async (file) => {
                var _a;
                const chunk = bundle[file];
                if (isRemaxEntry(chunk)) {
                    const filePath = Object.keys(chunk.modules)[0];
                    const page = pages.find(p => p === filePath);
                    if (page) {
                        const template = await createTemplate(file, options, meta);
                        templateAssets.push(template);
                        const config = createPageManifest(options, file, page, context);
                        const helperFile = await createHelperFile(file, meta);
                        if (helperFile) {
                            templateAssets.push(helperFile);
                        }
                        if (config) {
                            if (((_a = this.cache.get(file)) === null || _a === void 0 ? void 0 : _a.toString()) === config.source.toString()) {
                                return;
                            }
                            this.cache.set(file, config.source);
                            templateAssets.push(config);
                        }
                    }
                }
            }));
            templateAssets.forEach(file => {
                this.emitFile(Object.assign({ type: 'asset' }, file));
            });
        },
    };
}
exports.default = template;
