"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const API_1 = __importDefault(require("../API"));
const output_1 = require("./utils/output");
const path_1 = __importDefault(require("path"));
const plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
const plugin_commonjs_1 = __importDefault(require("@rollup/plugin-commonjs"));
const babel_1 = __importDefault(require("./plugins/babel"));
const rollup_plugin_url_1 = __importDefault(require("@remax/rollup-plugin-url"));
const plugin_json_1 = __importDefault(require("@rollup/plugin-json"));
const rollup_plugin_postcss_1 = __importDefault(require("@remax/rollup-plugin-postcss"));
const postcssUrl_1 = __importDefault(require("./plugins/postcssUrl"));
const rollup_plugin_progress_1 = __importDefault(require("rollup-plugin-progress"));
const rollup_plugin_delete_1 = __importDefault(require("rollup-plugin-delete"));
const rollup_plugin_copy_1 = __importDefault(require("rollup-plugin-copy"));
const stub_1 = __importDefault(require("./plugins/stub"));
const postcss_px2units_1 = __importDefault(require("@remax/postcss-px2units"));
const getEntries_1 = __importDefault(require("../getEntries"));
const getCssModuleConfig_1 = __importDefault(require("../getCssModuleConfig"));
const page_1 = __importDefault(require("./plugins/page"));
const removeSrc_1 = __importDefault(require("./plugins/removeSrc"));
const rename_1 = __importDefault(require("./plugins/rename"));
const plugin_replace_1 = __importDefault(require("@rollup/plugin-replace"));
const app_1 = __importDefault(require("./plugins/app"));
const named_exports_db_1 = __importDefault(require("named-exports-db"));
const fixRegeneratorRuntime_1 = __importDefault(require("./plugins/fixRegeneratorRuntime"));
const babelPlugin_1 = __importDefault(require("./plugins/nativeComponents/babelPlugin"));
const nativeComponents_1 = __importDefault(require("./plugins/nativeComponents"));
const components_1 = __importDefault(require("./plugins/components"));
const template_1 = __importDefault(require("./plugins/template"));
const alias_1 = __importDefault(require("./plugins/alias"));
const extensions_1 = __importDefault(require("../extensions"));
const lodash_1 = require("lodash");
const acorn_jsx_1 = __importDefault(require("acorn-jsx"));
const stringifyHostComponents_1 = __importDefault(require("./stringifyHostComponents"));
function rollupConfig(options, argv, context) {
    const stubModules = [];
    ['wechat', 'alipay', 'toutiao'].forEach(name => {
        if (API_1.default.adapter.name !== name) {
            stubModules.push(`${name}/esm/api`);
            stubModules.push(`${name}/esm/hostComponents`);
        }
    });
    const entries = getEntries_1.default(options, context);
    const cssModuleConfig = getCssModuleConfig_1.default(options.cssModules);
    // 获取 postcss 配置
    const postcssConfig = Object.assign({ options: {}, plugins: [] }, options.postcss);
    const envReplacement = {
        NODE_ENV: process.env.NODE_ENV || 'development',
        REMAX_PLATFORM: argv.target,
        REMAX_DEBUG: process.env.REMAX_DEBUG,
        REMAX_PX2RPX: `${options.pxToRpx}`,
        REMAX_HOST_COMPONENTS: () => stringifyHostComponents_1.default(),
    };
    Object.keys(process.env).forEach(k => {
        if (k.startsWith('REMAX_APP_')) {
            envReplacement[`${k}`] = process.env[k];
        }
    });
    const plugins = [
        rollup_plugin_copy_1.default({
            targets: [
                {
                    src: [`${options.rootDir}/native/*`],
                    dest: options.output,
                },
            ],
            copyOnce: true,
        }),
        alias_1.default(options),
        rollup_plugin_url_1.default({
            limit: 0,
            fileName: '[dirname][name][extname]',
            publicPath: '/',
            sourceDir: path_1.default.resolve(options.cwd, options.rootDir),
            include: ['**/*.svg', '**/*.png', '**/*.jpg', '**/*.jpeg', '**/*.gif'],
        }),
        plugin_json_1.default(),
        plugin_node_resolve_1.default({
            dedupe: [
                'react',
                'object-assign',
                'prop-types',
                'scheduler',
                'react-reconciler',
            ],
            extensions: extensions_1.default,
            customResolveOptions: {
                moduleDirectory: 'node_modules',
            },
        }),
        plugin_commonjs_1.default({
            include: /node_modules/,
            namedExports: named_exports_db_1.default,
            extensions: extensions_1.default,
            ignoreGlobal: false,
        }),
        stub_1.default({
            modules: stubModules,
        }),
        babel_1.default({
            include: entries.pages,
            extensions: lodash_1.without(extensions_1.default, '.json'),
            usePlugins: [babelPlugin_1.default(options), page_1.default],
            reactPreset: false,
        }),
        babel_1.default({
            include: entries.app,
            extensions: lodash_1.without(extensions_1.default, '.json'),
            usePlugins: [babelPlugin_1.default(options), app_1.default],
            reactPreset: false,
        }),
        babel_1.default({
            extensions: lodash_1.without(extensions_1.default, '.json'),
            usePlugins: [babelPlugin_1.default(options), components_1.default()],
            reactPreset: true,
        }),
        rollup_plugin_postcss_1.default(Object.assign(Object.assign({ extract: true }, postcssConfig.options), { modules: cssModuleConfig, plugins: [options.pxToRpx && postcss_px2units_1.default(), postcssUrl_1.default(options)]
                .filter(Boolean)
                .concat(postcssConfig.plugins) })),
        plugin_replace_1.default({
            values: Object.keys(envReplacement).reduce((acc, key) => {
                if (typeof envReplacement[key] === 'function') {
                    acc[`process.env.${key}`] = envReplacement[key];
                }
                else {
                    acc[`process.env.${key}`] = JSON.stringify(envReplacement[key]);
                }
                return acc;
            }, {}),
        }),
        rename_1.default({
            include: `${options.rootDir}/**`,
            map: input => {
                if (!input) {
                    return input;
                }
                input = input
                    // typescript
                    .replace(/\.ts$/, '.js')
                    .replace(/\.tsx$/, '.js')
                    // image
                    .replace(/\.png$/, '.png.js')
                    .replace(/\.gif$/, '.gif.js')
                    .replace(/\.svg$/, '.svg.js')
                    .replace(/\.jpeg$/, '.jpeg.js')
                    .replace(/\.jpg$/, '.jpg.js');
                // 不启用 css module 的 css 文件以及 app.css
                if (cssModuleConfig.globalModulePaths.some(reg => reg.test(input)) ||
                    input.indexOf('app.css') !== -1) {
                    return input.replace(/\.css/, API_1.default.getMeta().style);
                }
                return input.replace(/\.css/, '.css.js');
            },
        }),
        rename_1.default({
            matchAll: true,
            map: input => {
                return (input &&
                    input
                        // npm 包可能会有 jsx
                        .replace(/\.jsx$/, '.js')
                        // npm 包里可能会有 css
                        .replace(/\.less$/, '.less.js')
                        .replace(/\.sass$/, '.sass.js')
                        .replace(/\.scss$/, '.scss.js')
                        .replace(/\.styl$/, '.styl.js')
                        .replace(/node_modules/g, 'npm')
                        .replace(/\.js_commonjs-proxy$/, '.js_commonjs-proxy.js')
                        // 支付宝小程序不允许目录带 @
                        .replace(/@/g, '_'));
            },
        }),
        removeSrc_1.default(options),
        fixRegeneratorRuntime_1.default(),
        nativeComponents_1.default(options, entries.pages),
        template_1.default(options, context),
    ];
    /* istanbul ignore next */
    if (options.progress) {
        plugins.push(rollup_plugin_progress_1.default());
    }
    if (process.env.NODE_ENV === 'production') {
        plugins.unshift(rollup_plugin_delete_1.default({
            targets: [path_1.default.join(options.output, '*'), '!.tea'],
        }));
    }
    let config = {
        treeshake: process.env.NODE_ENV === 'production',
        input: [entries.app, ...entries.pages, ...entries.images],
        output: {
            dir: options.output,
            format: 'cjs',
            exports: 'named',
            sourcemap: false,
            extend: true,
        },
        preserveModules: true,
        preserveSymlinks: true,
        acornInjectPlugins: [acorn_jsx_1.default()],
        /* istanbul ignore next */
        onwarn(warning, warn) {
            if (warning.code === 'THIS_IS_UNDEFINED')
                return;
            if (warning.code === 'CIRCULAR_DEPENDENCY') {
                output_1.output('⚠️ 检测到循环依赖，如果不影响项目运行，请忽略', 'yellow');
            }
            if (!warning.message) {
                output_1.output(`⚠️ ${warning.code}:${warning.plugin || ''} ${warning.text}`, 'yellow');
            }
            else {
                output_1.output('⚠️ ' + warning.toString(), 'yellow');
            }
        },
        plugins,
    };
    if (typeof options.rollupOptions === 'function') {
        config = options.rollupOptions(config);
    }
    else if (options.rollupOptions) {
        config = Object.assign(Object.assign({}, config), options.rollupOptions);
    }
    config = API_1.default.extendsRollupConfig({ rollupConfig: config });
    return config;
}
exports.default = rollupConfig;
