"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar_1 = __importDefault(require("chokidar"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const rollup_1 = require("rollup");
const output_1 = require("../utils/output");
const checkChokidar_1 = require("../utils/checkChokidar");
const helpers_1 = require("./helpers");
const index_1 = __importDefault(require("../index"));
let isBundleRunning = false;
let isFirstRunWatcher = true;
// chokidar config
const chokidarConfig = {
    usePolling: true,
};
let extraFilesWatcher;
let watcher;
function runWather(remaxOptions, rollupOptions, cli, context) {
    if (isBundleRunning) {
        return;
    }
    const rollupWatchFiles = [`${remaxOptions.rootDir}/**`];
    // 配置重新build的路径
    const extraFiles = [
        `${remaxOptions.rootDir}/app.config.js`,
        `${remaxOptions.rootDir}/**/*.config.js`,
        `${remaxOptions.rootDir}/app.config.ts`,
        `${remaxOptions.rootDir}/**/*.config.ts`,
        `${remaxOptions.rootDir}/native`,
        `${remaxOptions.rootDir}/native/**`,
    ];
    isBundleRunning = true;
    watcher = rollup_1.watch([
        Object.assign(Object.assign({}, rollupOptions), { watch: {
                chokidar: checkChokidar_1.checkChokidar() && chokidarConfig,
                include: rollupWatchFiles,
            } }),
    ]);
    const watchEventHandle = (event) => {
        switch (event.code) {
            case 'START':
                output_1.output('🚚 编译...', 'blue');
                break;
            case 'END':
                isBundleRunning = false;
                output_1.output('💡 完成', 'green');
                break;
            case 'ERROR':
            case 'FATAL':
                isBundleRunning = false;
                const { error } = event;
                const name = error.code === 'PLUGIN_ERROR' ? error.plugin : error.code;
                output_1.output(`\n🚨 [${name}]: ${error.message}`, 'red');
                throw error;
            default:
                break;
        }
    };
    watcher.on('event', watchEventHandle);
    if (isFirstRunWatcher) {
        isFirstRunWatcher = false;
        console.log('\x1b[34m%s\x1b[0m', '🚀 启动 watch');
    }
    const close = (err) => {
        if (watcher) {
            watcher.close();
            watcher = null;
        }
        if (extraFilesWatcher) {
            extraFilesWatcher.close();
            watcher = null;
        }
        process.removeListener('uncaughtException', close);
        if (err) {
            console.error(err);
            process.exit(1);
        }
    };
    process.on('uncaughtException', close);
    // 监听额外的文件
    extraFilesWatcher = chokidar_1.default.watch(extraFiles);
    const reloadWatcher = (eventName) => (fileName) => {
        if (isFirstRunWatcher || isBundleRunning)
            return;
        if (helpers_1.isConfigFile(fileName)) {
            close();
            index_1.default(cli, context);
        }
        if (helpers_1.isInNativeDir(remaxOptions, fileName)) {
            const srcPath = path.join(remaxOptions.cwd, fileName);
            const destPath = path.join(remaxOptions.cwd, remaxOptions.output, fileName.replace(new RegExp(`^${remaxOptions.rootDir}/native`), ''));
            switch (eventName) {
                case 'change':
                case 'add':
                    fs.writeFileSync(destPath, fs.readFileSync(srcPath));
                    break;
                case 'addDir':
                    fs.mkdirSync(destPath);
                    break;
                case 'unlinkDir':
                    fs.rmdirSync(destPath);
                    break;
                case 'unlink':
                    fs.unlinkSync(destPath);
                    break;
            }
        }
    };
    extraFilesWatcher
        .on('addDir', reloadWatcher('addDir'))
        .on('add', reloadWatcher('add'))
        .on('unlink', reloadWatcher('unlink'))
        .on('unlinkDir', reloadWatcher('unlinkDir'))
        .on('change', reloadWatcher('change'));
}
exports.default = runWather;
