"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const semver_1 = __importDefault(require("semver"));
const chalk_1 = __importDefault(require("chalk"));
const winPath_1 = __importDefault(require("./winPath"));
const generatorStar = (count, symbol = '*') => {
    let starStr = '';
    for (let i = 0; i < count; i++) {
        starStr += symbol;
    }
    return starStr;
};
const generatorPlace = (origin, replace, longLength) => {
    const index = Math.round(origin.length / 2) - Math.round(longLength / 2) - 1;
    const length = replace.length;
    const originArray = origin.split('');
    const replaceArray = replace.split('');
    for (let i = 0; i < length; i++) {
        originArray[i + index] = replaceArray[i];
    }
    return originArray.join('');
};
exports.checkRemaxVersion = () => {
    try {
        const cliPackagePath = require.resolve('../package.json');
        const remaxPackagePath = winPath_1.default(path_1.default.join(process.cwd(), 'node_modules', 'remax', 'package.json'));
        if (fs_1.default.existsSync(remaxPackagePath)) {
            const remaxPkgConfig = require(remaxPackagePath);
            const cliPkgConfig = require(cliPackagePath);
            const remaxVersion = remaxPkgConfig.version;
            if (semver_1.default.neq(remaxVersion, cliPkgConfig.version)) {
                const placeholder = '*                                                     *';
                const origin = '*                                                               *';
                const remax = 'remax:    ' + chalk_1.default.red(remaxVersion);
                const remaxCLI = 'remax-cli: ' + chalk_1.default.green(cliPkgConfig.version);
                const longLength = remax.length > remaxCLI.length ? remax.length : remaxCLI.length;
                const warning = chalk_1.default.yellow(`remax-cli 版本与 remax 版本不匹配`);
                const upgrade = chalk_1.default.yellow(`请升级 remax 版本`);
                console.log(generatorStar(55));
                console.log(placeholder);
                console.log('*', `         ${warning}         `, '*');
                console.log('*', `                 ${upgrade}                 `, '*');
                console.log(placeholder);
                console.log(generatorPlace(origin, remax, longLength));
                console.log(generatorPlace(origin, remaxCLI, longLength));
                console.log(placeholder);
                console.log(generatorStar(55));
                return false;
            }
        }
        return true;
    }
    catch (err) {
        return false;
    }
};
