"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const API_1 = __importDefault(require("./API"));
const readManifest_1 = __importDefault(require("./readManifest"));
const output_1 = require("./build/utils/output");
function searchFile(file, strict) {
    const exts = ['ts', 'tsx', 'js', 'jsx'];
    for (const e of exts) {
        const extFile = file + '.' + e;
        if (fs_1.default.existsSync(extFile)) {
            return extFile;
        }
    }
    if (strict) {
        output_1.output(`\n🚨 [配置]: ${file} 不存在，请检查你的配置文件`, 'red');
    }
    return '';
}
exports.searchFile = searchFile;
exports.getAppConfig = (options) => {
    const appConfigPath = path_1.default.join(options.cwd, options.rootDir, 'app.config');
    return readManifest_1.default(appConfigPath, API_1.default.adapter.name, true);
};
// TODO: getEntries 处理 context 的逻辑要去掉
function getEntries(options, context) {
    var _a, _b, _c, _d, _e, _f;
    let pages = [];
    let subpackages = [];
    const appConfig = exports.getAppConfig(options);
    if (context) {
        pages = ((_b = (_a = context) === null || _a === void 0 ? void 0 : _a.pages) === null || _b === void 0 ? void 0 : _b.map(p => p.path)) || [];
        subpackages = ((_d = (_c = context) === null || _c === void 0 ? void 0 : _c.app) === null || _d === void 0 ? void 0 : _d.subpackages) || ((_f = (_e = context) === null || _e === void 0 ? void 0 : _e.app) === null || _f === void 0 ? void 0 : _f.subPackages) || [];
    }
    const entries = {
        app: searchFile(path_1.default.join(options.cwd, options.rootDir, 'app'), true),
        pages: [],
        images: [],
    };
    entries.pages = pages.reduce((ret, page) => {
        return [
            ...ret,
            searchFile(path_1.default.join(options.cwd, options.rootDir, page), true),
        ].filter(page => !!page);
    }, []);
    subpackages.forEach((pack) => {
        entries.pages = entries.pages.concat(pack.pages.reduce((ret, page) => {
            return [
                ...ret,
                searchFile(path_1.default.join(options.cwd, options.rootDir, pack.root, page), true),
            ].filter(page => !!page);
        }, []));
    });
    return API_1.default.getEntries(entries, appConfig, options);
}
exports.default = getEntries;
